@@ -38,31 +38,37 @@
 #include "gdata-types.h"
 #include "gdata-service.h"
 #include "gdata-private.h"
-#include "gdata-atom.h"
+#include "atom/gdata-category.h"
+#include "atom/gdata-link.h"
+#include "atom/gdata-author.h"
 
+static void gdata_entry_dispose (GObject *object);
 static void gdata_entry_finalize (GObject *object);
 static void gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static gboolean pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error);
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
 static gboolean post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error);
-static void real_get_xml (GDataEntry *self, GString *xml_string);
-static void real_get_namespaces (GDataEntry *self, GHashTable *namespaces);
+static void pre_get_xml (GDataParsable *parsable, GString *xml_string);
+static void get_xml (GDataParsable *parsable, GString *xml_string);
+static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
 
 struct _GDataEntryPrivate {
 	gchar *title;
+	gchar *summary;
 	gchar *id;
 	gchar *etag;
 	GTimeVal updated;
 	GTimeVal published;
-	GList *categories;
+	GList *categories; /* GDataCategory */
 	gchar *content;
-	GList *links;
-	GList *authors;
+	GList *links; /* GDataLink */
+	GList *authors; /* GDataAuthor */
 };
 
 enum {
 	PROP_TITLE = 1,
+	PROP_SUMMARY,
 	PROP_ETAG,
 	PROP_ID,
 	PROP_UPDATED,
@@ -82,22 +88,41 @@ gdata_entry_class_init (GDataEntryClass *klass)
 
 	g_type_class_add_private (klass, sizeof (GDataEntryPrivate));
 
-	gobject_class->set_property = gdata_entry_set_property;
 	gobject_class->get_property = gdata_entry_get_property;
+	gobject_class->set_property = gdata_entry_set_property;
+	gobject_class->dispose = gdata_entry_dispose;
 	gobject_class->finalize = gdata_entry_finalize;
 
 	parsable_class->pre_parse_xml = pre_parse_xml;
 	parsable_class->parse_xml = parse_xml;
 	parsable_class->post_parse_xml = post_parse_xml;
-
-	klass->get_xml = real_get_xml;
-	klass->get_namespaces = real_get_namespaces;
+	parsable_class->pre_get_xml = pre_get_xml;
+	parsable_class->get_xml = get_xml;
+	parsable_class->get_namespaces = get_namespaces;
+	parsable_class->element_name = "entry";
 
 	g_object_class_install_property (gobject_class, PROP_TITLE,
 				g_param_spec_string ("title",
 					"Title", "The title for this entry.",
 					NULL,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataEntry:summary:
+	 *
+	 * A short summary, abstract, or excerpt of the entry.
+	 *
+	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.summary">
+	 * Atom specification</ulink>.
+	 *
+	 * Since: 0.4.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_SUMMARY,
+				g_param_spec_string ("summary",
+					"Summary", "A short summary, abstract, or excerpt of the entry.",
+					NULL,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (gobject_class, PROP_ID,
 				g_param_spec_string ("id",
 					"ID", "The ID for this entry.",
@@ -137,21 +162,43 @@ gdata_entry_init (GDataEntry *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_ENTRY, GDataEntryPrivate);
 }
 
+static void
+gdata_entry_dispose (GObject *object)
+{
+	GDataEntryPrivate *priv = GDATA_ENTRY (object)->priv;
+
+	if (priv->categories != NULL) {
+		g_list_foreach (priv->categories, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->categories);
+	}
+	priv->categories = NULL;
+
+	if (priv->links != NULL) {
+		g_list_foreach (priv->links, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->links);
+	}
+	priv->links = NULL;
+
+	if (priv->authors != NULL) {
+		g_list_foreach (priv->authors, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->authors);
+	}
+	priv->authors = NULL;
+
+	/* Chain up to the parent class */
+	G_OBJECT_CLASS (gdata_entry_parent_class)->dispose (object);
+}
+
 static void
 gdata_entry_finalize (GObject *object)
 {
 	GDataEntryPrivate *priv = GDATA_ENTRY_GET_PRIVATE (object);
 
 	g_free (priv->title);
+	g_free (priv->summary);
 	xmlFree (priv->id);
 	xmlFree (priv->etag);
-	g_list_foreach (priv->categories, (GFunc) gdata_category_free, NULL);
-	g_list_free (priv->categories);
 	g_free (priv->content);
-	g_list_foreach (priv->links, (GFunc) gdata_link_free, NULL);
-	g_list_free (priv->links);
-	g_list_foreach (priv->authors, (GFunc) gdata_author_free, NULL);
-	g_list_free (priv->authors);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_entry_parent_class)->finalize (object);
@@ -166,6 +213,9 @@ gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GPa
 		case PROP_TITLE:
 			g_value_set_string (value, priv->title);
 			break;
+		case PROP_SUMMARY:
+			g_value_set_string (value, priv->summary);
+			break;
 		case PROP_ID:
 			g_value_set_string (value, priv->id);
 			break;
@@ -208,6 +258,9 @@ gdata_entry_set_property (GObject *object, guint property_id, const GValue *valu
 		case PROP_TITLE:
 			gdata_entry_set_title (self, g_value_get_string (value));
 			break;
+		case PROP_SUMMARY:
+			gdata_entry_set_summary (self, g_value_get_string (value));
+			break;
 		case PROP_CONTENT:
 			gdata_entry_set_content (self, g_value_get_string (value));
 			break;
@@ -263,7 +316,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		updated = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) updated, &(self->priv->updated)) == FALSE) {
 			/* Error */
-			gdata_parser_error_not_iso8601_format ("updated", "entry", (gchar*) updated, error);
+			gdata_parser_error_not_iso8601_format (node, (gchar*) updated, error);
 			xmlFree (updated);
 			return FALSE;
 		}
@@ -275,26 +328,18 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		published = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) published, &(self->priv->published)) == FALSE) {
 			/* Error */
-			gdata_parser_error_not_iso8601_format ("published", "entry", (gchar*) published, error);
+			gdata_parser_error_not_iso8601_format (node, (gchar*) published, error);
 			xmlFree (published);
 			return FALSE;
 		}
 		xmlFree (published);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
 		/* atom:category */
-		xmlChar *scheme, *term, *label;
-		GDataCategory *category;
+		GDataCategory *category = GDATA_CATEGORY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_CATEGORY, doc, node, NULL, error));
+		if (category == NULL)
+			return FALSE;
 
-		scheme = xmlGetProp (node, (xmlChar*) "scheme");
-		term = xmlGetProp (node, (xmlChar*) "term");
-		label = xmlGetProp (node, (xmlChar*) "label");
-
-		category = gdata_category_new ((gchar*) term, (gchar*) scheme, (gchar*) label);
 		self->priv->categories = g_list_prepend (self->priv->categories, category);
-
-		xmlFree (scheme);
-		xmlFree (term);
-		xmlFree (label);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "content") == 0) {
 		/* atom:content */
 		xmlChar *content = xmlNodeListGetString (doc, node->children, TRUE);
@@ -304,62 +349,23 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		xmlFree (content);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
 		/* atom:link */
-		xmlChar *href, *rel, *type, *hreflang, *title, *length;
-		gint length_int;
-		GDataLink *link;
+		GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
+		if (link == NULL)
+			return FALSE;
 
-		href = xmlGetProp (node, (xmlChar*) "href");
-		rel = xmlGetProp (node, (xmlChar*) "rel");
-		type = xmlGetProp (node, (xmlChar*) "type");
-		hreflang = xmlGetProp (node, (xmlChar*) "hreflang");
-		title = xmlGetProp (node, (xmlChar*) "title");
-		length = xmlGetProp (node, (xmlChar*) "length");
-
-		if (length == NULL)
-			length_int = -1;
-		else
-			length_int = strtoul ((gchar*) length, NULL, 10);
-
-		link = gdata_link_new ((gchar*) href, (gchar*) rel, (gchar*) type, (gchar*) hreflang, (gchar*) title, length_int);
 		self->priv->links = g_list_prepend (self->priv->links, link);
-
-		xmlFree (href);
-		xmlFree (rel);
-		xmlFree (type);
-		xmlFree (hreflang);
-		xmlFree (title);
-		xmlFree (length);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "author") == 0) {
 		/* atom:author */
-		GDataAuthor *author;
-		xmlNode *author_node;
-		xmlChar *name = NULL, *uri = NULL, *email = NULL;
+		GDataAuthor *author = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
+		if (author == NULL)
+			return FALSE;
 
-		author_node = node->children;
-		while (author_node != NULL) {
-			if (xmlStrcmp (author_node->name, (xmlChar*) "name") == 0) {
-				name = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else if (xmlStrcmp (author_node->name, (xmlChar*) "uri") == 0) {
-				uri = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else if (xmlStrcmp (author_node->name, (xmlChar*) "email") == 0) {
-				email = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else {
-				gdata_parser_error_unhandled_element ((gchar*) author_node->ns->prefix, (gchar*) author_node->name, "author", error);
-				xmlFree (name);
-				xmlFree (uri);
-				xmlFree (email);
-				return FALSE;
-			}
-
-			author_node = author_node->next;
-		}
-
-		author = gdata_author_new ((gchar*) name, (gchar*) uri, (gchar*) email);
 		self->priv->authors = g_list_prepend (self->priv->authors, author);
-
-		xmlFree (name);
-		xmlFree (uri);
-		xmlFree (email);
+	} else if (xmlStrcmp (node->name, (xmlChar*) "summary") == 0) {
+		/* atom:summary */
+		xmlChar *summary = xmlNodeListGetString (doc, node->children, TRUE);
+		gdata_entry_set_summary (self, (gchar*) summary);
+		xmlFree (summary);
 	} else if (GDATA_PARSABLE_CLASS (gdata_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
 		/* Error! */
 		return FALSE;
@@ -390,19 +396,20 @@ post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error)
 	return TRUE;
 }
 
-GDataEntry *
-_gdata_entry_new_from_xml (GType entry_type, const gchar *xml, gint length, GError **error)
+static void
+pre_get_xml (GDataParsable *parsable, GString *xml_string)
 {
-	g_return_val_if_fail (xml != NULL, NULL);
-	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY) == TRUE, FALSE);
+	GDataEntryPrivate *priv = GDATA_ENTRY (parsable)->priv;
 
-	return GDATA_ENTRY (_gdata_parsable_new_from_xml (entry_type, "entry", xml, length, NULL, error));
+	/* Add the entry's ETag, if available */
+	if (priv->etag != NULL)
+		g_string_append_printf (xml_string, " gd:etag='%s'", priv->etag);
 }
 
 static void
-real_get_xml (GDataEntry *self, GString *xml_string)
+get_xml (GDataParsable *parsable, GString *xml_string)
 {
-	GDataEntryPrivate *priv = self->priv;
+	GDataEntryPrivate *priv = GDATA_ENTRY (parsable)->priv;
 	gchar *title;
 	GList *categories, *links, *authors;
 
@@ -425,79 +432,30 @@ real_get_xml (GDataEntry *self, GString *xml_string)
 		g_free (published);
 	}
 
+	if (priv->summary != NULL) {
+		gchar *summary = g_markup_escape_text (priv->summary, -1);
+		g_string_append_printf (xml_string, "<summary type='text'>%s</summary>", summary);
+		g_free (summary);
+	}
+
 	if (priv->content != NULL) {
 		gchar *content = g_markup_escape_text (priv->content, -1);
 		g_string_append_printf (xml_string, "<content type='text'>%s</content>", content);
 		g_free (content);
 	}
 
-	for (categories = priv->categories; categories != NULL; categories = categories->next) {
-		GDataCategory *category = (GDataCategory*) categories->data;
+	for (categories = priv->categories; categories != NULL; categories = categories->next)
+		_gdata_parsable_get_xml (GDATA_PARSABLE (categories->data), xml_string, FALSE);
 
-		g_string_append_printf (xml_string, "<category term='%s'", category->term);
+	for (links = priv->links; links != NULL; links = links->next)
+		_gdata_parsable_get_xml (GDATA_PARSABLE (links->data), xml_string, FALSE);
 
-		if (G_LIKELY (category->scheme != NULL))
-			g_string_append_printf (xml_string, " scheme='%s'", category->scheme);
-
-		if (G_UNLIKELY (category->label != NULL)) {
-			gchar *label = g_markup_escape_text (category->label, -1);
-			g_string_append_printf (xml_string, " label='%s'", label);
-			g_free (label);
-		}
-
-		g_string_append (xml_string, "/>");
-	}
-
-	for (links = priv->links; links != NULL; links = links->next) {
-		GDataLink *link = (GDataLink*) links->data;
-
-		g_string_append_printf (xml_string, "<link href='%s'", link->href);
-
-		if (G_UNLIKELY (link->title != NULL)) {
-			gchar *link_title = g_markup_escape_text (link->title, -1);
-			g_string_append_printf (xml_string, " title='%s'", link_title);
-			g_free (link_title);
-		}
-
-		if (G_LIKELY (link->rel != NULL))
-			g_string_append_printf (xml_string, " rel='%s'", link->rel);
-		if (G_LIKELY (link->type != NULL))
-			g_string_append_printf (xml_string, " type='%s'", link->type);
-		if (G_UNLIKELY (link->hreflang != NULL))
-			g_string_append_printf (xml_string, " hreflang='%s'", link->hreflang);
-		if (G_UNLIKELY (link->length != -1))
-			g_string_append_printf (xml_string, " length='%i'", link->length);
-		g_string_append (xml_string, "/>");
-	}
-
-	for (authors = priv->authors; authors != NULL; authors = authors->next) {
-		GDataAuthor *author = (GDataAuthor*) authors->data;
-
-		gchar *name = g_markup_escape_text (author->name, -1);
-		g_string_append_printf (xml_string, "<author><name>%s</name>", name);
-		g_free (name);
-
-		if (G_LIKELY (author->uri != NULL)) {
-			gchar *uri = g_markup_escape_text (author->uri, -1);
-			g_string_append_printf (xml_string, "<uri>%s</uri>", uri);
-			g_free (uri);
-		}
-
-		if (G_UNLIKELY (author->email != NULL)) {
-			gchar *email = g_markup_escape_text (author->email, -1);
-			g_string_append_printf (xml_string, "<email>%s</email>", email);
-			g_free (email);
-		}
-
-		g_string_append (xml_string, "</author>");
-	}
-
-	if (_gdata_parsable_get_extra_xml (GDATA_PARSABLE (self)) != NULL)
-		g_string_append (xml_string, _gdata_parsable_get_extra_xml (GDATA_PARSABLE (self)));
+	for (authors = priv->authors; authors != NULL; authors = authors->next)
+		_gdata_parsable_get_xml (GDATA_PARSABLE (authors->data), xml_string, FALSE);
 }
 
 static void
-real_get_namespaces (GDataEntry *self, GHashTable *namespaces)
+get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
 	g_hash_table_insert (namespaces, (gchar*) "gd", (gchar*) "http://schemas.google.com/g/2005");
 }
@@ -516,26 +474,6 @@ gdata_entry_new (const gchar *id)
 	return g_object_new (GDATA_TYPE_ENTRY, "id", id, NULL);
 }
 
-/**
- * gdata_entry_new_from_xml:
- * @xml: the XML for just the entry, with full namespace declarations
- * @length: the length of @xml, or -1
- * @error: a #GError, or %NULL
- *
- * Creates a new #GDataEntry from the provided @xml.
- *
- * If @length is -1, @xml will be assumed to be nul-terminated.
- *
- * If an error occurs during parsing, a suitable error from #GDataParserError will be returned.
- *
- * Return value: a new #GDataEntry, or %NULL
- **/
-GDataEntry *
-gdata_entry_new_from_xml (const gchar *xml, gint length, GError **error)
-{
-	return GDATA_ENTRY (_gdata_parsable_new_from_xml (GDATA_TYPE_ENTRY, "entry", xml, length, NULL, error));
-}
-
 /**
  * gdata_entry_get_title:
  * @self: a #GDataEntry
@@ -567,6 +505,41 @@ gdata_entry_set_title (GDataEntry *self, const gchar *title)
 	g_object_notify (G_OBJECT (self), "title");
 }
 
+/**
+ * gdata_entry_get_summary:
+ * @self: a #GDataEntry
+ *
+ * Returns the summary of the entry.
+ *
+ * Return value: the entry's summary, or %NULL
+ *
+ * Since: 0.4.0
+ **/
+const gchar *
+gdata_entry_get_summary (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	return self->priv->summary;
+}
+
+/**
+ * gdata_entry_set_summary:
+ * @self: a #GDataEntry
+ * @summary: the new entry summary, or %NULL
+ *
+ * Sets the summary of the entry.
+ *
+ * Since: 0.4.0
+ **/
+void
+gdata_entry_set_summary (GDataEntry *self, const gchar *summary)
+{
+	g_return_if_fail (GDATA_IS_ENTRY (self));
+	g_free (self->priv->summary);
+	self->priv->summary = g_strdup (summary);
+	g_object_notify (G_OBJECT (self), "summary");
+}
+
 /**
  * gdata_entry_get_id:
  * @self: a #GDataEntry
@@ -635,15 +608,18 @@ gdata_entry_get_published (GDataEntry *self, GTimeVal *published)
  * @self: a #GDataEntry
  * @category: a #GDataCategory to add
  *
- * Adds @category to the list of categories in the given #GDataEntry.
+ * Adds @category to the list of categories in the given #GDataEntry, and increments its reference count.
+ *
+ * Duplicate categories will not be added to the list.
  **/
 void
 gdata_entry_add_category (GDataEntry *self, GDataCategory *category)
 {
 	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (category != NULL);
+	g_return_if_fail (GDATA_IS_CATEGORY (category));
 
-	self->priv->categories = g_list_prepend (self->priv->categories, category);
+	if (g_list_find_custom (self->priv->categories, category, (GCompareFunc) gdata_category_compare) == NULL)
+		self->priv->categories = g_list_prepend (self->priv->categories, g_object_ref (category));
 }
 
 /**
@@ -700,22 +676,25 @@ gdata_entry_set_content (GDataEntry *self, const gchar *content)
  * @self: a #GDataEntry
  * @link: a #GDataLink to add
  *
- * Adds @link to the list of links in the given #GDataEntry.
+ * Adds @link to the list of links in the given #GDataEntry and increments its reference count.
+ *
+ * Duplicate links will not be added to the list.
  **/
 void
 gdata_entry_add_link (GDataEntry *self, GDataLink *link)
 {
 	/* TODO: More link API */
 	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (link != NULL);
+	g_return_if_fail (GDATA_IS_LINK (link));
 
-	self->priv->links = g_list_prepend (self->priv->links, link);
+	if (g_list_find_custom (self->priv->links, link, (GCompareFunc) gdata_link_compare) == NULL)
+		self->priv->links = g_list_prepend (self->priv->links, g_object_ref (link));
 }
 
 static gint
 link_compare_cb (const GDataLink *link, const gchar *rel)
 {
-	return strcmp (link->rel, rel);
+	return strcmp (gdata_link_get_relation_type ((GDataLink*) link), rel);
 }
 
 /**
@@ -723,7 +702,10 @@ link_compare_cb (const GDataLink *link, const gchar *rel)
  * @self: a #GDataEntry
  * @rel: the value of the <structfield>rel</structfield> attribute of the desired link
  *
- * Looks up a link by <structfield>rel</structfield> value from the list of links in the entry.
+ * Looks up a link by relation type from the list of links in the entry. If the link has one of the standard Atom relation types,
+ * use one of the defined @rel values, instead of a static string. e.g. %GDATA_LINK_EDIT or %GDATA_LINK_SELF.
+ *
+ * In the rare event of requiring a list of links with the same @rel value, use gdata_entry_look_up_links().
  *
  * Return value: a #GDataLink, or %NULL if one was not found
  *
@@ -740,7 +722,38 @@ gdata_entry_look_up_link (GDataEntry *self, const gchar *rel)
 	element = g_list_find_custom (self->priv->links, rel, (GCompareFunc) link_compare_cb);
 	if (element == NULL)
 		return NULL;
-	return (GDataLink*) (element->data);
+	return GDATA_LINK (element->data);
+}
+
+/**
+ * gdata_entry_look_up_links:
+ * @self: a #GDataEntry
+ * @rel: the value of the <structfield>rel</structfield> attribute of the desired links
+ *
+ * Looks up a list of links by relation type from the list of links in the entry. If the links have one of the standard Atom
+ * relation types, use one of the defined @rel values, instead of a static string. e.g. %GDATA_LINK_EDIT or %GDATA_LINK_SELF.
+ *
+ * If you will only use the first link found, consider calling gdata_entry_look_up_link() instead.
+ *
+ * Return value: a #GList of #GDataLink<!-- -->s, or %NULL if none were found; free the list with g_list_free()
+ *
+ * Since: 0.4.0
+ **/
+GList *
+gdata_entry_look_up_links (GDataEntry *self, const gchar *rel)
+{
+	GList *i, *results = NULL;
+
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	g_return_val_if_fail (rel != NULL, NULL);
+
+	for (i = self->priv->links; i != NULL; i = i->next) {
+		const gchar *relation_type = gdata_link_get_relation_type (((GDataLink*) i->data));
+		if (strcmp (relation_type, rel) == 0)
+			results = g_list_prepend (results, i->data);
+	}
+
+	return g_list_reverse (results);
 }
 
 /**
@@ -748,7 +761,9 @@ gdata_entry_look_up_link (GDataEntry *self, const gchar *rel)
  * @self: a #GDataEntry
  * @author: a #GDataAuthor to add
  *
- * Adds @author to the list of authors in the given #GDataEntry.
+ * Adds @author to the list of authors in the given #GDataEntry and increments its reference count.
+ *
+ * Duplicate authors will not be added to the list.
  **/
 void
 gdata_entry_add_author (GDataEntry *self, GDataAuthor *author)
@@ -757,7 +772,8 @@ gdata_entry_add_author (GDataEntry *self, GDataAuthor *author)
 	g_return_if_fail (GDATA_IS_ENTRY (self));
 	g_return_if_fail (author != NULL);
 
-	self->priv->authors = g_list_prepend (self->priv->authors, author);
+	if (g_list_find_custom (self->priv->authors, author, (GCompareFunc) gdata_author_compare) == NULL)
+		self->priv->authors = g_list_prepend (self->priv->authors, g_object_ref (author));
 }
 
 /**
@@ -779,66 +795,3 @@ gdata_entry_is_inserted (GDataEntry *self)
 		return TRUE;
 	return FALSE;
 }
-
-static void
-build_namespaces_cb (gchar *prefix, gchar *href, GString *output)
-{
-	g_string_append_printf (output, " xmlns:%s='%s'", prefix, href);
-}
-
-static gboolean
-filter_namespaces_cb (gchar *prefix, gchar *href, GHashTable *canonical_namespaces)
-{
-	if (g_hash_table_lookup (canonical_namespaces, prefix) != NULL)
-		return TRUE;
-	return FALSE;
-}
-
-/**
- * gdata_entry_get_xml:
- * @self: a #GDataEntry
- *
- * Builds an XML representation of the #GDataEntry in its current state, such that it could be inserted on the server.
- * The XML is guaranteed to have all its namespaces declared properly in a self-contained fashion. The root element is
- * guaranteed to be <literal>&lt;entry&gt;</literal>.
- *
- * Return value: the entry's XML; free with g_free()
- **/
-gchar *
-gdata_entry_get_xml (GDataEntry *self)
-{
-	GDataEntryClass *klass;
-	GString *xml_string;
-	GHashTable *namespaces, *extra_namespaces;
-
-	klass = GDATA_ENTRY_GET_CLASS (self);
-	g_assert (klass->get_xml != NULL);
-	g_assert (klass->get_namespaces != NULL);
-
-	/* Get the namespaces the class uses */
-	namespaces = g_hash_table_new (g_str_hash, g_str_equal);
-	klass->get_namespaces (self, namespaces);
-	extra_namespaces = _gdata_parsable_get_extra_namespaces (GDATA_PARSABLE (self));
-
-	/* Remove any duplicate extra namespaces */
-	g_hash_table_foreach_remove (extra_namespaces, (GHRFunc) filter_namespaces_cb, namespaces);
-
-	/* Build up the namespace list */
-	xml_string = g_string_new ("<entry xmlns='http://www.w3.org/2005/Atom'");
-	g_hash_table_foreach (namespaces, (GHFunc) build_namespaces_cb, xml_string);
-	g_hash_table_foreach (extra_namespaces, (GHFunc) build_namespaces_cb, xml_string);
-
-	/* Add the entry's ETag, if available */
-	if (self->priv->etag != NULL)
-		g_string_append_printf (xml_string, " gd:etag='%s'>", self->priv->etag);
-	else
-		g_string_append_c (xml_string, '>');
-
-	g_hash_table_destroy (namespaces);
-
-	/* Add the rest of the XML */
-	klass->get_xml (self, xml_string);
-	g_string_append (xml_string, "</entry>");
-
-	return g_string_free (xml_string, FALSE);
-}
