@@ -124,7 +124,7 @@ gdata_service_class_init (GDataServiceClass *klass)
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CLIENT_ID,
 				g_param_spec_string ("client-id",
-					"Client ID", "A client ID for your application (see http://code.google.com/apis/youtube/2.0/developers_guide_protocol_api_query_parameters.html#clientsp).",
+					"Client ID", "A client ID for your application.",
 					NULL,
 					G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -336,19 +336,23 @@ real_parse_error_response (GDataService *self, GDataServiceError error_type, gui
 	switch (status) {
 		case 400:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+				     /* Translators: the parameter is an error message returned by the server. */
 				     _("Invalid request URI or header, or unsupported nonstandard parameter: %s"), reason_phrase);
 			return;
 		case 401:
 		case 403:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+				     /* Translators: the parameter is an error message returned by the server. */
 				     _("Authentication required: %s"), reason_phrase);
 			return;
 		case 404:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_NOT_FOUND,
+				     /* Translators: the parameter is an error message returned by the server. */
 				     _("The requested resource was not found: %s"), reason_phrase);
 			return;
 		case 409:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT,
+				     /* Translators: the parameter is an error message returned by the server. */
 				     _("The entry has been modified since it was downloaded: %s"), reason_phrase);
 			return;
 		case 500:
@@ -361,18 +365,22 @@ real_parse_error_response (GDataService *self, GDataServiceError error_type, gui
 	switch (error_type) {
 		case GDATA_SERVICE_ERROR_WITH_INSERTION:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_INSERTION,
+				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
 				     _("Error code %u when inserting an entry: %s"), status, reason_phrase);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_UPDATE:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPDATE,
+				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
 				     _("Error code %u when updating an entry: %s"), status, reason_phrase);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_DELETION:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DELETION,
+				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
 				     _("Error code %u when deleting an entry: %s"), status, reason_phrase);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_QUERY:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_QUERY,
+				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
 				     _("Error code %u when querying: %s"), status, reason_phrase);
 			break;
 		default:
@@ -657,26 +665,32 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 
 		if (strncmp (error_start, "NotVerified", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_NOT_VERIFIED,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("Your account's e-mail address has not been verified. (%s)"), uri);
 			goto login_error;
 		} else if (strncmp (error_start, "TermsNotAgreed", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_TERMS_NOT_AGREED,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("You have not agreed to the service's terms and conditions. (%s)"), uri);
 			goto login_error;
 		} else if (strncmp (error_start, "AccountDeleted", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_ACCOUNT_DELETED,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("This account has been deleted. (%s)"), uri);
 			goto login_error;
 		} else if (strncmp (error_start, "AccountDisabled", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_ACCOUNT_DISABLED,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("This account has been disabled. (%s)"), uri);
 			goto login_error;
 		} else if (strncmp (error_start, "ServiceDisabled", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("This account's access to this service has been disabled. (%s)"), uri);
 			goto login_error;
 		} else if (strncmp (error_start, "ServiceUnavailable", error_end - error_start) == 0) {
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_UNAVAILABLE,
+				     /* Translators: the parameter is a URI for further information. */
 				     _("This service is not available at the moment. (%s)"), uri);
 			goto login_error;
 		}
@@ -790,6 +804,7 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GError **
 		if (new_uri == NULL) {
 			gchar *uri_string = soup_uri_to_string (new_uri, FALSE);
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+				     /* Translators: the parameter is the URI which is invalid. */
 				     _("Invalid redirect URI: %s"), uri_string);
 			g_free (uri_string);
 			return SOUP_STATUS_NONE;
@@ -881,6 +896,7 @@ gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery
 	g_return_if_fail (GDATA_IS_SERVICE (self));
 	g_return_if_fail (feed_uri != NULL);
 	g_return_if_fail (entry_type != G_TYPE_INVALID);
+	g_return_if_fail (callback != NULL);
 
 	data = g_slice_new (QueryAsyncData);
 	data->feed_uri = g_strdup (feed_uri);
@@ -926,53 +942,18 @@ gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GErr
 	g_assert_not_reached ();
 }
 
-/**
- * gdata_service_query:
- * @self: a #GDataService
- * @feed_uri: the feed URI to query, including the host name and protocol
- * @query: a #GDataQuery with the query parameters, or %NULL
- * @entry_type: a #GType for the #GDataEntry<!-- -->s to build from the XML
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
- * @error: a #GError, or %NULL
- *
- * Queries the service's @feed_uri feed to build a #GDataFeed.
- *
- * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
- *
- * A %GDATA_SERVICE_ERROR_WITH_QUERY will be returned if the server indicates there is a problem with the query, but subclasses may override
- * this and return their own errors. See their documentation for more details.
- *
- * For each entry in the response feed, @progress_callback will be called in the main thread. If there was an error parsing the XML response,
- * a #GDataParserError will be returned.
- *
- * If the query is successful and the feed supports pagination, @query will be updated with the pagination URIs, and the next or previous page
- * can then be loaded by calling gdata_query_next_page() or gdata_query_previous_page() before running the query again.
- *
- * If the #GDataQuery's ETag is set and it finds a match on the server, %FALSE will be returned, but @error will remain unset. Otherwise,
- * @query's ETag will be updated with the ETag from the returned feed, if available.
- *
- * Return value: a #GDataFeed of query results, or %NULL; unref with g_object_unref()
- **/
-GDataFeed *
-gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
-		     GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
+/* Does the bulk of the work of gdata_service_query. Split out because certain queries (such as that done by
+ * gdata_youtube_service_query_single_video()) only return a single entry, and thus need special parsing code. */
+SoupMessage *
+_gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GCancellable *cancellable,
+		      GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	GDataServiceClass *klass;
-	GDataFeed *feed;
 	SoupMessage *message;
-	gchar *query_uri;
 	guint status;
-	GDataLink *link;
-
-	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
-	g_return_val_if_fail (feed_uri != NULL, NULL);
-	g_return_val_if_fail (entry_type != G_TYPE_INVALID, NULL);
 
 	if (query != NULL) {
-		query_uri = gdata_query_get_query_uri (query, feed_uri);
+		gchar *query_uri = gdata_query_get_query_uri (query, feed_uri);
 		message = soup_message_new (SOUP_METHOD_GET, query_uri);
 		g_free (query_uri);
 	} else {
@@ -1010,24 +991,76 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 		return NULL;
 	}
 
+	return message;
+}
+
+/**
+ * gdata_service_query:
+ * @self: a #GDataService
+ * @feed_uri: the feed URI to query, including the host name and protocol
+ * @query: a #GDataQuery with the query parameters, or %NULL
+ * @entry_type: a #GType for the #GDataEntry<!-- -->s to build from the XML
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: data to pass to the @progress_callback function
+ * @error: a #GError, or %NULL
+ *
+ * Queries the service's @feed_uri feed to build a #GDataFeed.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
+ * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ *
+ * A %GDATA_SERVICE_ERROR_WITH_QUERY will be returned if the server indicates there is a problem with the query, but subclasses may override
+ * this and return their own errors. See their documentation for more details.
+ *
+ * For each entry in the response feed, @progress_callback will be called in the main thread. If there was an error parsing the XML response,
+ * a #GDataParserError will be returned.
+ *
+ * If the query is successful and the feed supports pagination, @query will be updated with the pagination URIs, and the next or previous page
+ * can then be loaded by calling gdata_query_next_page() or gdata_query_previous_page() before running the query again.
+ *
+ * If the #GDataQuery's ETag is set and it finds a match on the server, %FALSE will be returned, but @error will remain unset. Otherwise,
+ * @query's ETag will be updated with the ETag from the returned feed, if available.
+ *
+ * Return value: a #GDataFeed of query results, or %NULL; unref with g_object_unref()
+ **/
+GDataFeed *
+gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
+		     GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
+{
+	GDataServiceClass *klass;
+	GDataFeed *feed;
+	SoupMessage *message;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (feed_uri != NULL, NULL);
+	g_return_val_if_fail (entry_type != G_TYPE_INVALID, NULL);
+
+	message = _gdata_service_query (self, feed_uri, query, cancellable, progress_callback, progress_user_data, error);
 	g_assert (message->response_body->data != NULL);
 
+	klass = GDATA_SERVICE_GET_CLASS (self);
 	feed = _gdata_feed_new_from_xml (klass->feed_type, message->response_body->data, message->response_body->length, entry_type,
 					 progress_callback, progress_user_data, error);
 	g_object_unref (message);
 
+	if (feed == NULL)
+		return NULL;
+
 	/* Update the query with the feed's ETag */
-	if (query != NULL && gdata_feed_get_etag (feed) != NULL)
+	if (query != NULL && feed != NULL && gdata_feed_get_etag (feed) != NULL)
 		gdata_query_set_etag (query, gdata_feed_get_etag (feed));
 
 	/* Update the query with the next and previous URIs from the feed */
-	if (query != NULL) {
+	if (query != NULL && feed != NULL) {
+		GDataLink *link;
+
 		link = gdata_feed_look_up_link (feed, "next");
 		if (link != NULL)
-			_gdata_query_set_next_uri (query, link->href);
+			_gdata_query_set_next_uri (query, gdata_link_get_uri (link));
 		link = gdata_feed_look_up_link (feed, "previous");
 		if (link != NULL)
-			_gdata_query_set_previous_uri (query, link->href);
+			_gdata_query_set_previous_uri (query, gdata_link_get_uri (link));
 	}
 
 	return feed;
@@ -1200,7 +1233,7 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 		klass->append_query_headers (self, message);
 
 	/* Append the data */
-	upload_data = gdata_entry_get_xml (entry);
+	upload_data = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
 	soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
 
 	/* Send the message */
@@ -1229,7 +1262,8 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 	g_assert (message->response_body->data != NULL);
 
 	/* Parse the XML; create and return a new GDataEntry of the same type as @entry */
-	updated_entry = _gdata_entry_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length, error);
+	updated_entry = GDATA_ENTRY (gdata_parsable_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length,
+								  error));
 	g_object_unref (message);
 
 	return updated_entry;
@@ -1362,9 +1396,9 @@ gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable
 	g_return_val_if_fail (GDATA_IS_ENTRY (entry), NULL);
 
 	/* Get the edit URI */
-	link = gdata_entry_look_up_link (entry, "edit");
+	link = gdata_entry_look_up_link (entry, GDATA_LINK_EDIT);
 	g_assert (link != NULL);
-	message = soup_message_new (SOUP_METHOD_PUT, link->href);
+	message = soup_message_new (SOUP_METHOD_PUT, gdata_link_get_uri (link));
 
 	/* Make sure subclasses set their headers */
 	klass = GDATA_SERVICE_GET_CLASS (self);
@@ -1376,7 +1410,7 @@ gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable
 		soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (entry));
 
 	/* Append the data */
-	upload_data = gdata_entry_get_xml (entry);
+	upload_data = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
 	soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
 
 	/* Send the message */
@@ -1405,7 +1439,8 @@ gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable
 	g_assert (message->response_body->data != NULL);
 
 	/* Parse the XML; create and return a new GDataEntry of the same type as @entry */
-	updated_entry = _gdata_entry_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length, error);
+	updated_entry = GDATA_ENTRY (gdata_parsable_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length,
+								  error));
 	g_object_unref (message);
 
 	return updated_entry;
@@ -1529,9 +1564,9 @@ gdata_service_delete_entry (GDataService *self, GDataEntry *entry, GCancellable
 	g_return_val_if_fail (GDATA_IS_ENTRY (entry), FALSE);
 
 	/* Get the edit URI */
-	link = gdata_entry_look_up_link (entry, "edit");
+	link = gdata_entry_look_up_link (entry, GDATA_LINK_EDIT);
 	g_assert (link != NULL);
-	message = soup_message_new (SOUP_METHOD_DELETE, link->href);
+	message = soup_message_new (SOUP_METHOD_DELETE, gdata_link_get_uri (link));
 
 	/* Make sure subclasses set their headers */
 	klass = GDATA_SERVICE_GET_CLASS (self);
