@@ -38,16 +38,16 @@
 #include "gdata-calendar-event.h"
 #include "gdata-private.h"
 #include "gdata-service.h"
-#include "gdata-gdata.h"
 #include "gdata-parser.h"
 #include "gdata-types.h"
 
+static void gdata_calendar_event_dispose (GObject *object);
 static void gdata_calendar_event_finalize (GObject *object);
 static void gdata_calendar_event_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_calendar_event_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
-static void get_xml (GDataEntry *entry, GString *xml_string);
+static void get_xml (GDataParsable *parsable, GString *xml_string);
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
-static void get_namespaces (GDataEntry *entry, GHashTable *namespaces);
+static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
 
 struct _GDataCalendarEventPrivate {
 	GTimeVal edited;
@@ -93,18 +93,17 @@ gdata_calendar_event_class_init (GDataCalendarEventClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
-	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataCalendarEventPrivate));
 
-	gobject_class->set_property = gdata_calendar_event_set_property;
 	gobject_class->get_property = gdata_calendar_event_get_property;
+	gobject_class->set_property = gdata_calendar_event_set_property;
+	gobject_class->dispose = gdata_calendar_event_dispose;
 	gobject_class->finalize = gdata_calendar_event_finalize;
 
 	parsable_class->parse_xml = parse_xml;
-
-	entry_class->get_xml = get_xml;
-	entry_class->get_namespaces = get_namespaces;
+	parsable_class->get_xml = get_xml;
+	parsable_class->get_namespaces = get_namespaces;
 
 	/**
 	 * GDataCalendarEvent:edited:
@@ -292,21 +291,42 @@ gdata_calendar_event_init (GDataCalendarEvent *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_CALENDAR_EVENT, GDataCalendarEventPrivate);
 }
 
+static void
+gdata_calendar_event_dispose (GObject *object)
+{
+	GDataCalendarEventPrivate *priv = GDATA_CALENDAR_EVENT (object)->priv;
+
+	if (priv->times != NULL) {
+		g_list_foreach (priv->times, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->times);
+	}
+	priv->times = NULL;
+
+	if (priv->people != NULL) {
+		g_list_foreach (priv->people, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->people);
+	}
+	priv->people = NULL;
+
+	if (priv->places != NULL) {
+		g_list_foreach (priv->places, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->places);
+	}
+	priv->places = NULL;
+
+	/* Chain up to the parent class */
+	G_OBJECT_CLASS (gdata_calendar_event_parent_class)->dispose (object);
+}
+
 static void
 gdata_calendar_event_finalize (GObject *object)
 {
-	GDataCalendarEventPrivate *priv = GDATA_CALENDAR_EVENT_GET_PRIVATE (object);
+	GDataCalendarEventPrivate *priv = GDATA_CALENDAR_EVENT (object)->priv;
 
 	g_free (priv->status);
 	g_free (priv->visibility);
 	g_free (priv->transparency);
 	g_free (priv->uid);
-	g_list_foreach (priv->times, (GFunc) gdata_gd_when_free, NULL);
-	g_list_free (priv->times);
-	g_list_foreach (priv->people, (GFunc) gdata_gd_who_free, NULL);
-	g_list_free (priv->people);
-	g_list_foreach (priv->places, (GFunc) gdata_gd_where_free, NULL);
-	g_list_free (priv->places);
 	g_free (priv->recurrence);
 	xmlFree ((xmlChar*) priv->original_event_id);
 	xmlFree ((xmlChar*) priv->original_event_uri);
@@ -424,25 +444,6 @@ gdata_calendar_event_new (const gchar *id)
 	return g_object_new (GDATA_TYPE_CALENDAR_EVENT, "id", id, NULL);
 }
 
-/**
- * gdata_calendar_event_new_from_xml:
- * @xml: an XML string
- * @length: the length in characters of @xml, or %-1
- * @error: a #GError, or %NULL
- *
- * Creates a new #GDataCalendarEvent from an XML string. If @length is %-1, the length of
- * the string will be calculated.
- *
- * Errors from #GDataParserError can be returned if problems are found in the XML.
- *
- * Return value: a new #GDataCalendarEvent, or %NULL; unref with g_object_unref()
- **/
-GDataCalendarEvent *
-gdata_calendar_event_new_from_xml (const gchar *xml, gint length, GError **error)
-{
-	return GDATA_CALENDAR_EVENT (_gdata_entry_new_from_xml (GDATA_TYPE_CALENDAR_EVENT, xml, length, error));
-}
-
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
@@ -459,7 +460,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
 			/* Error */
-			gdata_parser_error_not_iso8601_format ("app:edited", "entry", (gchar*) edited, error);
+			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
 			xmlFree (edited);
 			return FALSE;
 		}
@@ -494,28 +495,28 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		/* gd:eventStatus */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gd:eventStatus", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_status (self, (gchar*) value);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "visibility") == 0) {
 		/* gd:visibility */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gd:visibility", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_visibility (self, (gchar*) value);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "transparency") == 0) {
 		/* gd:transparency */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gd:transparency", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_transparency (self, (gchar*) value);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "uid") == 0) {
 		/* gCal:uid */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:uid", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_uid (self, (gchar*) value);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "sequence") == 0) {
@@ -525,7 +526,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 
 		value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:sequence", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		else
 			value_uint = strtoul ((gchar*) value, NULL, 10);
 		xmlFree (value);
@@ -533,107 +534,51 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		gdata_calendar_event_set_sequence (self, value_uint);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "when") == 0) {
 		/* gd:when */
-		xmlChar *start_time, *end_time, *value_string;
-		GTimeVal start_time_timeval, end_time_timeval;
-		GDataGDWhen *when;
-		gboolean is_date = FALSE;
-
-		/* Start time */
-		start_time = xmlGetProp (node, (xmlChar*) "startTime");
-		if (gdata_parser_time_val_from_date ((gchar*) start_time, &start_time_timeval) == TRUE) {
-			is_date = TRUE;
-		} else if (g_time_val_from_iso8601 ((gchar*) start_time, &start_time_timeval) == FALSE) {
-			/* Error */
-			gdata_parser_error_not_iso8601_format ("gd:when", "entry", (gchar*) start_time, error);
-			xmlFree (start_time);
+		GDataGDWhen *when = GDATA_GD_WHEN (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHEN, doc, node, NULL, error));
+		if (when == NULL)
 			return FALSE;
-		}
-		xmlFree (start_time);
-
-		/* End time (optional) */
-		end_time = xmlGetProp (node, (xmlChar*) "endTime");
-		if (end_time != NULL) {
-			gboolean success;
-
-			if (is_date == TRUE)
-				success = gdata_parser_time_val_from_date ((gchar*) end_time, &end_time_timeval);
-			else
-				success = g_time_val_from_iso8601 ((gchar*) end_time, &end_time_timeval);
-
-			if (success == FALSE) {
-				/* Error */
-				gdata_parser_error_not_iso8601_format ("gd:when", "entry", (gchar*) end_time, error);
-				xmlFree (end_time);
-				return FALSE;
-			}
-			xmlFree (end_time);
-		}
-
-		value_string = xmlGetProp (node, (xmlChar*) "value");
-		when = gdata_gd_when_new (&start_time_timeval, (end_time == NULL) ? NULL : &end_time_timeval, is_date, (gchar*) value_string, NULL);
-		xmlFree (value_string);
 
 		gdata_calendar_event_add_time (self, when);
-
-		/* TODO: Deal with reminders (<gd:reminder> child elements) */
 	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanModify") == 0) {
 		/* gCal:guestsCanModify */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:guestsCanModify", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_guests_can_modify (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanInviteOthers") == 0) {
 		/* gCal:guestsCanInviteOthers */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:guestsCanInviteOthers", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_guests_can_invite_others (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanSeeGuests") == 0) {
 		/* gCal:guestsCanSeeGuests */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:guestsCanSeeGuests", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_guests_can_see_guests (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "anyoneCanAddSelf") == 0) {
 		/* gCal:anyoneCanAddSelf */
 		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
 		if (value == NULL)
-			return gdata_parser_error_required_property_missing ("gCal:anyoneCanAddSelf", "value", error);
+			return gdata_parser_error_required_property_missing (node, "value", error);
 		gdata_calendar_event_set_anyone_can_add_self (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 		xmlFree (value);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "who") == 0) {
 		/* gd:who */
-		xmlChar *email, *rel, *value_string;
-		GDataGDWho *who;
-
-		rel = xmlGetProp (node, (xmlChar*) "rel");
-		value_string = xmlGetProp (node, (xmlChar*) "valueString");
-		email = xmlGetProp (node, (xmlChar*) "email");
-		/* TODO: deal with the attendeeType, attendeeStatus and entryLink */
-
-		who = gdata_gd_who_new ((gchar*) rel, (gchar*) value_string, (gchar*) email);
-		xmlFree (rel);
-		xmlFree (value_string);
-		xmlFree (email);
+		GDataGDWho *who = GDATA_GD_WHO (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHO, doc, node, NULL, error));
+		if (who == NULL)
+			return FALSE;
 
 		gdata_calendar_event_add_person (self, who);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
 		/* gd:where */
-		xmlChar *label, *rel, *value_string;
-		GDataGDWhere *where;
-
-		rel = xmlGetProp (node, (xmlChar*) "rel");
-		value_string = xmlGetProp (node, (xmlChar*) "valueString");
-		label = xmlGetProp (node, (xmlChar*) "label");
-		/* TODO: deal with the entryLink */
-
-		where = gdata_gd_where_new ((gchar*) rel, (gchar*) value_string, (gchar*) label);
-		xmlFree (rel);
-		xmlFree (value_string);
-		xmlFree (label);
+		GDataGDWhere *where = GDATA_GD_WHERE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHERE, doc, node, NULL, error));
+		if (where == NULL)
+			return FALSE;
 
 		gdata_calendar_event_add_place (self, where);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "recurrence") == 0) {
@@ -658,13 +603,21 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 }
 
 static void
-get_xml (GDataEntry *entry, GString *xml_string)
+get_child_xml (GList *list, GString *xml_string)
 {
-	GDataCalendarEventPrivate *priv = GDATA_CALENDAR_EVENT (entry)->priv;
 	GList *i;
 
+	for (i = list; i != NULL; i = i->next)
+		_gdata_parsable_get_xml (GDATA_PARSABLE (i->data), xml_string, FALSE);
+}
+
+static void
+get_xml (GDataParsable *parsable, GString *xml_string)
+{
+	GDataCalendarEventPrivate *priv = GDATA_CALENDAR_EVENT (parsable)->priv;
+
 	/* Chain up to the parent class */
-	GDATA_ENTRY_CLASS (gdata_calendar_event_parent_class)->get_xml (entry, xml_string);
+	GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->get_xml (parsable, xml_string);
 
 	/* Add all the Calendar-specific XML */
 
@@ -708,67 +661,9 @@ get_xml (GDataEntry *entry, GString *xml_string)
 	if (priv->recurrence != NULL)
 		g_string_append_printf (xml_string, "<gd:recurrence>%s</gd:recurrence>", priv->recurrence);
 
-	for (i = priv->times; i != NULL; i = i->next) {
-		gchar *start_time;
-		GDataGDWhen *when = (GDataGDWhen*) i->data;
-
-		if (when->is_date == TRUE)
-			start_time = gdata_parser_date_from_time_val (&(when->start_time));
-		else
-			start_time = g_time_val_to_iso8601 (&(when->start_time));
-
-		g_string_append_printf (xml_string, "<gd:when startTime='%s'", start_time);
-		g_free (start_time);
-
-		if (when->end_time.tv_sec != 0 || when->end_time.tv_usec != 0) {
-			gchar *end_time;
-
-			if (when->is_date == TRUE)
-				end_time = gdata_parser_date_from_time_val (&(when->end_time));
-			else
-				end_time = g_time_val_to_iso8601 (&(when->end_time));
-
-			g_string_append_printf (xml_string, " endTime='%s'", end_time);
-			g_free (end_time);
-		}
-
-		if (when->value_string != NULL)
-			g_string_append_printf (xml_string, " value='%s'", when->value_string);
-
-		g_string_append (xml_string, "/>");
-
-		/* TODO: Deal with reminders (<gd:reminder> child elements) */
-	}
-
-	for (i = priv->people; i != NULL; i = i->next) {
-		GDataGDWho *who = (GDataGDWho*) i->data;
-
-		g_string_append (xml_string, "<gd:who");
-		if (who->email != NULL)
-			g_string_append_printf (xml_string, " email='%s'", who->email);
-		if (who->rel != NULL)
-			g_string_append_printf (xml_string, " rel='%s'", who->rel);
-		if (who->value_string != NULL)
-			g_string_append_printf (xml_string, " valueString='%s'", who->value_string);
-		g_string_append (xml_string, "/>");
-
-		/* TODO: deal with the attendeeType, attendeeStatus and entryLink */
-	}
-
-	for (i = priv->places; i != NULL; i = i->next) {
-		GDataGDWhere *where = (GDataGDWhere*) i->data;
-
-		g_string_append (xml_string, "<gd:where");
-		if (where->label != NULL)
-			g_string_append_printf (xml_string, " label='%s'", where->label);
-		if (where->rel != NULL)
-			g_string_append_printf (xml_string, " rel='%s'", where->rel);
-		if (where->value_string != NULL)
-			g_string_append_printf (xml_string, " valueString='%s'", where->value_string);
-		g_string_append (xml_string, "/>");
-
-		/* TODO: deal with the entryLink */
-	}
+	get_child_xml (priv->times, xml_string);
+	get_child_xml (priv->people, xml_string);
+	get_child_xml (priv->places, xml_string);
 
 	/* TODO:
 	 * - Finish supporting all tags
@@ -780,10 +675,10 @@ get_xml (GDataEntry *entry, GString *xml_string)
 }
 
 static void
-get_namespaces (GDataEntry *entry, GHashTable *namespaces)
+get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
 	/* Chain up to the parent class */
-	GDATA_ENTRY_CLASS (gdata_calendar_event_parent_class)->get_namespaces (entry, namespaces);
+	GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->get_namespaces (parsable, namespaces);
 
 	g_hash_table_insert (namespaces, (gchar*) "gd", (gchar*) "http://schemas.google.com/g/2005");
 	g_hash_table_insert (namespaces, (gchar*) "gCal", (gchar*) "http://schemas.google.com/gCal/2005");
@@ -1102,8 +997,9 @@ gdata_calendar_event_set_anyone_can_add_self (GDataCalendarEvent *self, gboolean
  * @self: a #GDataCalendarEvent
  * @who: a #GDataGDWho to add
  *
- * Adds the person @who to the event as a guest (attendee, organiser, performer, etc.).
- * The #GDataCalendarEvent takes ownership of @who, so it must not be freed after being added.
+ * Adds the person @who to the event as a guest (attendee, organiser, performer, etc.), and increments its reference count.
+ *
+ * Duplicate people will not be added to the list.
  **/
 void
 gdata_calendar_event_add_person (GDataCalendarEvent *self, GDataGDWho *who)
@@ -1111,7 +1007,8 @@ gdata_calendar_event_add_person (GDataCalendarEvent *self, GDataGDWho *who)
 	g_return_if_fail (GDATA_IS_CALENDAR_EVENT (self));
 	g_return_if_fail (who != NULL);
 
-	self->priv->people = g_list_append (self->priv->people, who);
+	if (g_list_find_custom (self->priv->people, who, (GCompareFunc) gdata_gd_who_compare) == NULL)
+		self->priv->people = g_list_append (self->priv->people, g_object_ref (who));
 }
 
 /**
@@ -1136,8 +1033,9 @@ gdata_calendar_event_get_people (GDataCalendarEvent *self)
  * @self: a #GDataCalendarEvent
  * @where: a #GDataGDWhere to add
  *
- * Adds the place @where to the event as a location.
- * The #GDataCalendarEvent takes ownership of @where, so it must not be freed after being added.
+ * Adds the place @where to the event as a location and increments its reference count.
+ *
+ * Duplicate places will not be added to the list.
  **/
 void
 gdata_calendar_event_add_place (GDataCalendarEvent *self, GDataGDWhere *where)
@@ -1145,7 +1043,8 @@ gdata_calendar_event_add_place (GDataCalendarEvent *self, GDataGDWhere *where)
 	g_return_if_fail (GDATA_IS_CALENDAR_EVENT (self));
 	g_return_if_fail (where != NULL);
 
-	self->priv->places = g_list_append (self->priv->places, where);
+	if (g_list_find_custom (self->priv->places, where, (GCompareFunc) gdata_gd_where_compare) == NULL)
+		self->priv->places = g_list_append (self->priv->places, g_object_ref (where));
 }
 
 /**
@@ -1170,8 +1069,9 @@ gdata_calendar_event_get_places (GDataCalendarEvent *self)
  * @self: a #GDataCalendarEvent
  * @when: a #GDataGDWhen to add
  *
- * Adds @when to the event as a time period when the event happens.
- * The #GDataCalendarEvent takes ownership of @when, so it must not be freed after being added.
+ * Adds @when to the event as a time period when the event happens, and increments its reference count.
+ *
+ * Duplicate times will not be added to the list.
  *
  * Since: 0.2.0
  **/
@@ -1179,9 +1079,10 @@ void
 gdata_calendar_event_add_time (GDataCalendarEvent *self, GDataGDWhen *when)
 {
 	g_return_if_fail (GDATA_IS_CALENDAR_EVENT (self));
-	g_return_if_fail (when != NULL);
+	g_return_if_fail (GDATA_IS_GD_WHEN (when));
 
-	self->priv->times = g_list_append (self->priv->times, when);
+	if (g_list_find_custom (self->priv->times, when, (GCompareFunc) gdata_gd_when_compare) == NULL)
+		self->priv->times = g_list_append (self->priv->times, g_object_ref (when));
 }
 
 /**
@@ -1228,11 +1129,11 @@ gdata_calendar_event_get_primary_time (GDataCalendarEvent *self, GTimeVal *start
 	if (self->priv->times == NULL || self->priv->times->next != NULL)
 		return FALSE;
 
-	primary_when = (GDataGDWhen*) self->priv->times->data;
+	primary_when = GDATA_GD_WHEN (self->priv->times->data);
 	if (start_time != NULL)
-		*start_time = primary_when->start_time;
+		gdata_gd_when_get_start_time (primary_when, start_time);
 	if (end_time != NULL)
-		*end_time = primary_when->end_time;
+		gdata_gd_when_get_end_time (primary_when, end_time);
 	if (when != NULL)
 		*when = primary_when;
 
